@@ -7,6 +7,7 @@
 #include <hyprutils/memory/WeakPtr.hpp>
 #include <wayland-client.h>
 #include <xf86drmMode.h>
+#include <optional>
 
 namespace Aquamarine {
     class CDRMBackend;
@@ -14,6 +15,7 @@ namespace Aquamarine {
     class CDRMOutput;
     struct SDRMConnector;
     class CDRMRenderer;
+    class CDRMDumbAllocator;
 
     typedef std::function<void(void)> FIdleCallback;
 
@@ -162,6 +164,7 @@ namespace Aquamarine {
             bool     ownModeID = false;
             uint32_t modeID    = 0;
             uint32_t gammaLut  = 0;
+            uint32_t ctm       = 0;
         } atomic;
 
         Hyprutils::Memory::CSharedPointer<SDRMPlane> primary;
@@ -175,6 +178,7 @@ namespace Aquamarine {
                 uint32_t vrr_enabled;
                 uint32_t gamma_lut;
                 uint32_t gamma_lut_size;
+                uint32_t ctm;
 
                 // atomic-modesetting only
 
@@ -242,13 +246,16 @@ namespace Aquamarine {
         uint32_t                                  flags    = 0;
         bool                                      test     = false;
         drmModeModeInfo                           modeInfo;
+        std::optional<Hyprutils::Math::Mat3x3>    ctm;
 
         struct {
             uint32_t gammaLut = 0;
             uint32_t fbDamage = 0;
             uint32_t modeBlob = 0;
+            uint32_t ctmBlob  = 0;
             bool     blobbed  = false;
             bool     gammad   = false;
+            bool     ctmd     = false;
         } atomic;
 
         void calculateMode(Hyprutils::Memory::CSharedPointer<SDRMConnector> connector);
@@ -334,28 +341,29 @@ namespace Aquamarine {
     class CDRMBackend : public IBackendImplementation {
       public:
         virtual ~CDRMBackend();
-        virtual eBackendType                                            type();
-        virtual bool                                                    start();
-        virtual std::vector<Hyprutils::Memory::CSharedPointer<SPollFD>> pollFDs();
-        virtual int                                                     drmFD();
-        virtual bool                                                    dispatchEvents();
-        virtual uint32_t                                                capabilities();
-        virtual bool                                                    setCursor(Hyprutils::Memory::CSharedPointer<IBuffer> buffer, const Hyprutils::Math::Vector2D& hotspot);
-        virtual void                                                    onReady();
-        virtual std::vector<SDRMFormat>                                 getRenderFormats();
-        virtual std::vector<SDRMFormat>                                 getCursorFormats();
-        virtual bool                                                    createOutput(const std::string& name = "");
-        virtual Hyprutils::Memory::CSharedPointer<IAllocator>           preferredAllocator();
-        virtual std::vector<SDRMFormat>                                 getRenderableFormats();
+        virtual eBackendType                                               type();
+        virtual bool                                                       start();
+        virtual std::vector<Hyprutils::Memory::CSharedPointer<SPollFD>>    pollFDs();
+        virtual int                                                        drmFD();
+        virtual bool                                                       dispatchEvents();
+        virtual uint32_t                                                   capabilities();
+        virtual bool                                                       setCursor(Hyprutils::Memory::CSharedPointer<IBuffer> buffer, const Hyprutils::Math::Vector2D& hotspot);
+        virtual void                                                       onReady();
+        virtual std::vector<SDRMFormat>                                    getRenderFormats();
+        virtual std::vector<SDRMFormat>                                    getCursorFormats();
+        virtual bool                                                       createOutput(const std::string& name = "");
+        virtual Hyprutils::Memory::CSharedPointer<IAllocator>              preferredAllocator();
+        virtual std::vector<SDRMFormat>                                    getRenderableFormats();
+        virtual std::vector<Hyprutils::Memory::CSharedPointer<IAllocator>> getAllocators();
 
-        Hyprutils::Memory::CWeakPointer<CDRMBackend>                    self;
+        Hyprutils::Memory::CWeakPointer<CDRMBackend>                       self;
 
-        void                                                            log(eBackendLogLevel, const std::string&);
-        bool                                                            sessionActive();
-        int                                                             getNonMasterFD();
+        void                                                               log(eBackendLogLevel, const std::string&);
+        bool                                                               sessionActive();
+        int                                                                getNonMasterFD();
 
-        std::vector<FIdleCallback>                                      idleCallbacks;
-        std::string                                                     gpuName;
+        std::vector<FIdleCallback>                                         idleCallbacks;
+        std::string                                                        gpuName;
 
       private:
         CDRMBackend(Hyprutils::Memory::CSharedPointer<CBackend> backend);
@@ -390,6 +398,8 @@ namespace Aquamarine {
         std::vector<SDRMFormat>                                       formats;
         std::vector<SDRMFormat>                                       glFormats;
 
+        Hyprutils::Memory::CSharedPointer<CDRMDumbAllocator>          dumbAllocator;
+
         bool                                                          atomic = false;
 
         struct {
